/*
    list.c
        Implantacin de las funciones de la lista.
*/


#include "list.h"

static ListStatus listStatus;
const unsigned int MarkAlive = 0xBEEF;
const unsigned int MarkDead = 0xDEADBEEF;

ListStatus getListStatus()
{
    return listStatus;
}

bool isValidPos(List l, Position p)
{
    bool toret = isValidList( l ) && ( p != NULL || l->first == NULL );

    if ( toret
      && p != NULL ) {
        toret = toret && ( p->signature == MarkAlive );
    }

    return toret;
}


static inline
void setListNextPos(List l, Position pos, Position npos)
{
    if ( isValidPos( l, pos )
      && isValidPos( l, npos ) )
    {
            listStatus = ListOk;
            pos->next = npos;

            if ( pos == getListLastPos( l ) ) {
                l->last = npos;
            }
    } else  listStatus = ListInvalidPos;

    return;
}

static inline
void setListPreviousPos(List l, Position pos, Position npos)
{
    if ( isValidPos( l, pos )
      && isValidPos( l, npos ) )
    {
            listStatus = ListOk;
            pos->prev = npos;

            if ( pos == getListFirstPos( l ) ) {
                l->first = npos;
            }
    } else  listStatus = ListInvalidPos;

    return;
}

List createList()
{
    listStatus = ListOk;
    List toret = (List) malloc( sizeof( struct _List ) );

    if ( toret != NULL ) {
        toret->first = toret->last = NULL;
        toret->signature = MarkAlive;
    }

    return toret;
}

void destroyList(List l)
{
    listStatus = ListOk;
    deleteListRange( l, getListFirstPos( l ), getListLastPos( l ) );
    l->first = l->last = NULL;
    l->signature = MarkDead;
    free( l );
}

Position deleteListElement(List l, Position p)
{
    listStatus = ListOk;
    Position * previous;
    Position toret = NULL;

    if ( isValidPos( l, p ) ) {
            // Buscar el puntero que seala a la posicin
            if ( p != getListFirstPos( l ) )
                    previous = &( getListPreviousPos( l, p )->next );
            else    previous = &( l->first );

            // Marcar el siguiente como siguiente del anterior
            Position next = getListNextPos( l, p );
            *previous = next;

            // Marcar el anterior del siguiente como anterior del actual
            if ( next != NULL ) {
                next->prev = p->prev;
            }

            // Devolver
            toret = getListPreviousPos( l, p );
            if ( toret == NULL ) {
                toret = getListNextPos( l, p );
            }

            // Eliminar
            p->signature = MarkDead;
            free( p );
    } else listStatus = ListInvalidPos;

    return toret;
}

void deleteListRange(List l, Position beg, Position end)
{
    listStatus = ListOk;

    // Borrar hasta el final
    while( beg != NULL
        && beg != end )
    {
        beg = deleteListElement( l, beg );

        if ( getListStatus() != ListOk ) {
            beg = NULL;

            if ( getListStatus() == ListInvalidPos ) {
                listStatus = ListInvalidRange;
            }

            break;
        }
    }

    // Elimiar el ltimo
    if ( beg != NULL ) {
        deleteListElement( l, beg );
    }
}

Position getListNextPos(List l, Position pos)
{
    listStatus = ListOk;
    Position toret = NULL;

    if ( pos != NULL
      && isValidPos( l, pos ) )
    {
        toret = pos->next;
    }

    return toret;
}

inline
Position getListPreviousPos(List l, Position pos)
{
    Position toret = NULL;
    listStatus = ListOk;

    if ( pos != NULL
      && isValidPos( l, pos ) )
    {
        toret = pos->prev;
    }

    return toret;
}

Position insertListElement(List l, Position p, void * data)
{
    listStatus = ListOk;
    Position * previous;
    Position toret = (Position) malloc( sizeof( Node ) );

    if ( toret != NULL ) {
        if ( isValidPos( l, p ) ) {
                // Encontrar nodos siguiente y anterior
                if ( p != getListFirstPos( l ) )
                        previous = &( getListPreviousPos( l, p )->next );
                else    previous = &( l->first );

                toret->content = data;
                toret->signature = MarkAlive;

                // Asignar siguiente y previo en el nuevo nodo
                if ( getListFirstPos( l ) == NULL ) {
                    toret->prev = NULL;
                }
                toret->next = p;

                // Actualizar el nodo ltimo de la lista
                if ( p == NULL ) {
                    l->last = toret;
                }

                // Asignar los otros nodos colindantes
                *previous = toret;
                if ( p != NULL ) {
                    p->prev = toret;
                }
        } else listStatus = ListInvalidPos;
    } else listStatus = ListNoMemory;

    return toret;
}

void * getListPosContent(List l, Position p)
{
    void * toret = NULL;
    listStatus = ListInvalidPos;

    if ( isValidPos( l, p ) ) {
        toret = p->content;
        listStatus = ListOk;
    }

    return toret;
}

void setListPosContent(List l, Position p, void * data)
{
    listStatus = ListInvalidPos;

    if ( isValidPos( l, p ) ) {
        p->content = data;
        listStatus = ListOk;
    }
}

void advanceListPos(List l, Position &pos, int delta)
{
    listStatus = ListOk;
    bool atras = ( delta < 0 );
    delta = abs( delta );

    if ( isValidPos( l, pos ) ) {
        for(; delta > 0; --delta) {
            if ( atras )
                    pos = getListPreviousPos( l, pos );
            else    pos = getListNextPos( l, pos );
        }
    }

    return;
}
