/**
    list.h
        Mdulo de listas.
*/

#ifndef LIST_H_INCLUDED
#define LIST_H_INCLUDED


extern const unsigned int MarkAlive;
extern const unsigned int MarkDead;

#include <cstdlib>

typedef enum {
    ListOk, ListNoMemory, ListInvalidRange, ListInvalidPos
} ListStatus;

typedef struct _Node {
    unsigned int signature;
    void * content;
    struct _Node * next;
    struct _Node * prev;
} Node;

typedef Node * Position;

typedef struct _List {
    unsigned int signature;

    /** Primer nodo de la la lista **/
    Position first;
    /** ltimo nodo de la la lista **/
    Position last;
} * List;

/**
    Avanza una posicin recoriendo una lista
    @param l La lista a recorrer
    @param delta El nmero de posiciones a avanzar
    @param pos La posicin a hacer avanzar
*/
void advanceListPos(List l, Position &pos, int delta);

/**
    Devuelve verdadero o falso segn la lista sea correcta o no
    @param p La posicin en la lista a comprobar
    @param l La lista a comprobar
    @return true si es una posicin vlida, false en otro caso.
*/
inline
bool isValidList(List l)
    { return ( l != NULL && l->signature == MarkAlive ); }


/**
    Devuelve verdadero o falso segn la lista est vaca o no
    @param l La lista a comprobar
    @return true si est vaca, false en otro caso.
*/
inline
bool isListEmpty(List l)
    { return ( isValidList( l ) && l->first == NULL ); }

/** Crear una lista.
        @return la lista, ya preparada
*/
List createList();

/** Destruir una lista
        @param l La lista a destruir
*/
void destroyList(List l);

/** Borrar todos los elementos
        @param beg La primera posicin del rango a borrar
        @param end La ltima posicin del rango a borrar
        @param l La lista a la cual borrar todos los elementos
*/
void deleteListRange(List l, Position beg, Position end);

/** Borrar un elemento
        @param p La posicin del elemento a borrar
        @param l La lista a la cual borrar el elemento
        @return La nueva posicin donde estaba el anterior elemento
*/
Position deleteListElement(List l, Position pos);


/**
    Obtener el ltimo error de las operaciones en la lista
        @return El error, como enumerado ListStatus
        @see ListStatus
*/
ListStatus getListStatus();

/**
    Devuelve verdadero o falso segn la posicin p sea
    vlida en una lista (cualquier) o no.
    @param p La posicin en la lista a comprobar
    @param l La lista a comprobar
    @return true si es una posicin vlida, false en otro caso.
*/
bool isValidPos(List l, Position p);

/**
    Devuelve la ltima Posicin en una lista
    @param l La lista de la que obtener la ltima posicin
    @return El nodo correspondiente a la ltima posicin.
*/
inline
Position getListLastPos(List l)
    { return ( isValidList( l ) ? l->last : NULL ); }

/**
    Devuelve la primera posicin en una lista
    @param l La lista de la que obtener la primera posicin
    @return El nodo correspondiente a la primera posicin.
*/
inline
Position getListFirstPos(List l)
    { return ( isValidList( l ) ? l->first : NULL ); }

/**
    Devuelve la siguiente posicin en una lista
    @param l La lista de la que obtener la siguiente posicin
    @param p La posicin de la que obtener la siguiente posicin
    @return El nodo correspondiente a la posicin siguiente.
*/
Position getListNextPos(List l, Position pos);

/**
    Devuelve la anterior posicin en una lista
    @param l La lista de la que obtener la anterior posicin
    @param p La posicin de la que obtener la anterior posicin
    @return El nodo correspondiente a la posicin anterior.
*/
Position getListPreviousPos(List l, Position pos);

/**
    Devuelve el contenido de un nodo de la lista
    @param l La lista de la que obtener la anterior posicin
    @param p La posicin de la que obtener la anterior posicin
    @return El contenido del nodo correspondiente
*/
void * getListPosContent(List l, Position p);

/**
    Modifica el contenido de un nodo de la lista
    @param l La lista de la que obtener la anterior posicin
    @param p La posicin de la que obtener la anterior posicin
    @return El contenido del nodo correspondiente
*/
void setListPosContent(List l, Position p, void * data);

/**
    Cambia la siguiente posicin en una posicin de la lista
    @param l La lista de la que insertar la nueva posicin
    @param pos La posicin donde insertar el nuevo elemento
    @return La posicin del nuevo elemento
*/
Position insertListElement(List l, Position pos, void * data);

#endif // LIST_H_INCLUDED
